package cz.zweistein.df.soundsense.gui;

import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import cz.zweistein.df.soundsense.config.ConfigurationXML;
import cz.zweistein.df.soundsense.config.Sound;
import cz.zweistein.df.soundsense.config.SoundFile;
import cz.zweistein.df.soundsense.gui.adapter.ChannelListAdapterModel;
import cz.zweistein.df.soundsense.gui.adapter.ConfigurationXMLTreeAdapter;
import cz.zweistein.df.soundsense.gui.adapter.SFXListAdapter;
import cz.zweistein.df.soundsense.gui.control.SoundFileInfoPanel;
import cz.zweistein.df.soundsense.gui.control.SoundInfoPanel;
import cz.zweistein.df.soundsense.gui.control.StopPlaybackButtonEditor;
import cz.zweistein.df.soundsense.gui.control.StopPlaybackButtonRenderer;
import cz.zweistein.df.soundsense.gui.control.VolumeSlider;
import cz.zweistein.df.soundsense.output.sound.SoundProcesor;

public class Gui extends JPanel {
	private static final long serialVersionUID = 7326794456268877831L;
	
	public Gui(final ConfigurationXML config, final SoundProcesor sp) {
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		
        add(new VolumeSlider(config, sp));
        
        JTabbedPane tabPane = new JTabbedPane();
        
        add(tabPane);
        
        JPanel currentlyPlayingPanel = new JPanel();
        currentlyPlayingPanel.setLayout(new BoxLayout(currentlyPlayingPanel, BoxLayout.PAGE_AXIS));
        
        JLabel sfxListLabel = new JLabel("SFX:", JLabel.LEFT);
        currentlyPlayingPanel.add(sfxListLabel);
        
        JList sfxList = new JList(new SFXListAdapter(sp.getPlayerManager()));
        JScrollPane sfxListScroller = new JScrollPane(sfxList);
        currentlyPlayingPanel.add(sfxListScroller);
        
        JLabel chanelsListLabel = new JLabel("Channels:", JLabel.LEFT);
        currentlyPlayingPanel.add(chanelsListLabel);
        
        TableModel channelTableModel = new ChannelListAdapterModel(sp.getPlayerManager());
		final JTable channelTable = new JTable(channelTableModel) {
			private static final long serialVersionUID = 7878780961926946651L;

			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
		        if (column == 4) {
		            return new StopPlaybackButtonRenderer();
		        }
		        return super.getCellRenderer(row, column);
		    }

			@Override
			public TableCellEditor getCellEditor(int row, int column) {
				if (column == 4) {
		            return new StopPlaybackButtonEditor(sp.getPlayerManager());
		        }
				return super.getCellEditor(row, column);
			}
			
		};
		initColumnSizes(channelTable);
	
        JScrollPane channelListScroller = new JScrollPane(channelTable);
        currentlyPlayingPanel.add(channelListScroller);

        tabPane.addTab("Currently Playing", currentlyPlayingPanel);
        
        JPanel configurationPanel = new JPanel();
        configurationPanel.setLayout(new BorderLayout());
        
        JTree soundsTree = new JTree(new ConfigurationXMLTreeAdapter(sp.getSoundsXML()));
        
        JScrollPane soundsTreeScroller = new JScrollPane(soundsTree);
        configurationPanel.add(soundsTreeScroller, BorderLayout.CENTER);
        
        JPanel infos = new JPanel();
        infos.setLayout(new BoxLayout(infos, BoxLayout.PAGE_AXIS));
        configurationPanel.add(infos, BorderLayout.PAGE_END);
        
        final SoundInfoPanel soundInfoPanel = new SoundInfoPanel(sp.getPlayerManager());
        soundInfoPanel.setVisible(false);
        infos.add(soundInfoPanel);
        
        final SoundFileInfoPanel soundFileInfoPanel = new SoundFileInfoPanel(sp.getPlayerManager());
        soundFileInfoPanel.setVisible(false);
        infos.add(soundFileInfoPanel);
        
        soundsTree.addTreeSelectionListener(new TreeSelectionListener() {
        	@Override
        	public void valueChanged(TreeSelectionEvent e) {
        		
        		Object selectedItem = e.getNewLeadSelectionPath().getLastPathComponent();
        		
        		if (selectedItem instanceof Sound) {
        			
        			soundInfoPanel.setVisible(true);
        			soundFileInfoPanel.setVisible(false);
        			
        			soundInfoPanel.updateSound((Sound) selectedItem);
        			
        		} else if (selectedItem instanceof SoundFile) {
        			
        			soundInfoPanel.setVisible(false);
        			soundFileInfoPanel.setVisible(true);
        			
        			soundFileInfoPanel.updateSoundFile((SoundFile) selectedItem);
        			
        		} else if (selectedItem instanceof String) {
        			
        			soundInfoPanel.setVisible(false);
        			soundFileInfoPanel.setVisible(false);
        			
        		}
        	}
        });

        tabPane.addTab("Sounds Configuration", configurationPanel);

	}
	
    private static void createAndShowGUI(final ConfigurationXML config, SoundProcesor sp) {
        //Create and set up the window.
        JFrame frame = new JFrame("SoundSense");
        frame.setIconImage(Toolkit.getDefaultToolkit().getImage("icon.png"));
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        frame.addWindowListener(new WindowListener() {
			
			@Override
			public void windowClosed(WindowEvent arg0) {
			}

			@Override
			public void windowActivated(WindowEvent arg0) {
			}

			@Override
			public void windowClosing(WindowEvent arg0) {
				config.saveConfiguration();
			}

			@Override
			public void windowDeactivated(WindowEvent arg0) {
			}

			@Override
			public void windowDeiconified(WindowEvent arg0) {
			}

			@Override
			public void windowIconified(WindowEvent arg0) {
			}

			@Override
			public void windowOpened(WindowEvent arg0) {
			}

		});
        
        Gui gui = new Gui(config, sp);
                
        //Add content to the window.
        frame.add(gui, BorderLayout.CENTER);

        //Display the window.
        frame.pack();
        frame.setVisible(true); 
    }

    public static void newGui(final ConfigurationXML config, final SoundProcesor sp) {
	    SwingUtilities.invokeLater(new Runnable() {
	        public void run() {
	            createAndShowGUI(config, sp);
	        }
	    });
    }

    
    private void initColumnSizes(JTable table) {
        TableColumn column = table.getColumnModel().getColumn(4);
        
        column.setMinWidth(16);
        column.setMaxWidth(16);

    }

}
